/*
 ChibiOS - Copyright (C) 2006..2018 Giovanni Di Sirio
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 http://www.apache.org/licenses/LICENSE-2.0
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

#include "ch.h"
#include "hal.h"

#define ADC_GRP2_NUM_CHANNELS   2
#define ADC_GRP2_BUF_DEPTH      16

static adcsample_t samples[ADC_GRP2_NUM_CHANNELS * ADC_GRP2_BUF_DEPTH];

static float temperature[1];



/*
 * ADC streaming callback.
 */
static void adccallback(ADCDriver *adcp, adcsample_t *buffer, size_t n) {


  (void)adcp;

  int32_t tempAvg = 0;
  //Samples are alternating: temp, vrefint, temp, vrefint, ...
  for(int i = 0; i < (ADC_GRP2_BUF_DEPTH * ADC_GRP2_NUM_CHANNELS); i++) {
      tempAvg += samples[i];
  }
  tempAvg /= ADC_GRP2_BUF_DEPTH;


  //Convert in Volt: multiply for the reference voltage (3.3V) and divide for the resolution (2^12-1 bit)
  float temp_val_volt =(float) ((tempAvg * 3.3) / 4095);


  /*
   * Convert the voltage to Celsius
   * According to ST:
   *   slope=4.3 mV/C
   *   Voltage at 25C = 1.43V
   * Thus:
   *  temp [C] = (vtemp[V]-1.43 [V])/0,0043 [V]+25[C]
   */
  float temp_val_cels=((temp_val_volt-1.43)/0.0043)+25;
  temperature[0]=temp_val_cels;
  if( temperature[0]<300)
    palSetPad(GPIOE, GPIOE_LED6_GREEN);


}

static void adcerrorcallback(ADCDriver *adcp, adcerror_t err) {
  palSetPad(GPIOE, GPIOE_LED10_RED);

  (void)adcp;
  (void)err;
}

/*
 * ADC conversion group.
 * Mode:        Continuous (Circular), 1 channel, SW triggered.
 * Channels:    IN16 (Channel associated to the temperature sensor)
 */
static const ADCConversionGroup adcgrpcfg = {
                  TRUE,
                  ADC_GRP2_NUM_CHANNELS,
                  adccallback,
                  adcerrorcallback,
                  ADC_CFGR_CONT ,             /* CFGR1 */
                  ADC_TR(0, 1),
                  {                                                             /* SMPR[2]  */
                    0,
                    ADC_SMPR2_SMP_AN16(ADC_SMPR_SMP_61P5)
                       | ADC_SMPR2_SMP_AN17(ADC_SMPR_SMP_61P5)
                  },
                  {                                                             /* SQR[4]   */
                    0,
                    ADC_SQR2_SQ7_N(ADC_CHANNEL_IN16) | ADC_SQR2_SQ8_N(ADC_CHANNEL_IN17),
                    0,
                    0
                  }
};



/*
 * Application entry point.
 */
int main(void) {

  /*
   * System initializations.
   * - HAL initialization, this also initializes the configured device drivers
   *   and performs the board-specific initializations.
   * - Kernel initialization, the main() function becomes a thread and the
   *   RTOS is active.
   */
  halInit();
  chSysInit();
  /*
   * Activates the ADC1 driver and the temperature sensor.
   */
  adcStart(&ADCD1, NULL);
  adcSTM32EnableTS(&ADCD1);
  adcSTM32DisableVREF(&ADCD1);
  adcSTM32DisableVBAT(&ADCD1);

  palSetGroupMode(GPIOC, PAL_PORT_BIT(1) | PAL_PORT_BIT(2),
                  0, PAL_MODE_INPUT_ANALOG);

  /*
   * Starts an ADC continuous conversion.
   */
  adcStartConversionI(&ADCD1, &adcgrpcfg, samples, ADC_GRP2_BUF_DEPTH);


  /*
   * Normal main() thread activity, in this demo it does nothing.
   */
  while (true) {
   /* msg_t res=adcConvert(&ADCD1, &adcgrpcfg, samples, ADC_GRP2_BUF_DEPTH);
    if(res==MSG_OK)
      palSetPad(GPIOE, GPIOE_LED6_GREEN);
*/
    if (palReadPad(GPIOA, GPIOA_BUTTON)) {
      adcStopConversionI(&ADCD1);
    }

    chThdSleepMilliseconds(500);
  }
}

